PRAGMA journal_mode = WAL;

CREATE TABLE IF NOT EXISTS "device" (
	"id" INTEGER PRIMARY KEY AUTOINCREMENT,
	"mac" TEXT NOT NULL UNIQUE,
	"config_group_id" INTEGER NOT NULL,
	FOREIGN KEY("config_group_id") REFERENCES config_group("id") ON DELETE CASCADE ON UPDATE CASCADE
);

CREATE TABLE IF NOT EXISTS "interface" (
	"id" INTEGER PRIMARY KEY AUTOINCREMENT,
	"interface" TEXT NOT NULL,
	"config_group_id" INTEGER NOT NULL,
	FOREIGN KEY("config_group_id") REFERENCES config_group("id") ON DELETE CASCADE ON UPDATE CASCADE
);

CREATE TABLE IF NOT EXISTS "config_group" (
	"id" INTEGER PRIMARY KEY AUTOINCREMENT,
	"name" TEXT NOT NULL UNIQUE,
	"profile_id" INTEGER NOT NULL,
	"visible" INTEGER NOT NULL DEFAULT 1, -- 1=show, 0=hide
	"pause_expired" INTEGER, -- second from epoch
	FOREIGN KEY("profile_id") REFERENCES profile("id") ON DELETE CASCADE ON UPDATE CASCADE
);

CREATE TABLE IF NOT EXISTS "profile" (
	"id" INTEGER PRIMARY KEY AUTOINCREMENT,
	"name" TEXT NOT NULL UNIQUE,
	"visible" INTEGER NOT NULL DEFAULT 1, -- 1=show, 0=hide
	"permission" INTEGER NOT NULL DEFAULT 0, /* 0=Normal, 1=Grant permission to unlock block page */
	"safe_search_bing" INTEGER NOT NULL DEFAULT 0, /* 0, 1 */
	"safe_search_google" INTEGER NOT NULL DEFAULT 0, /* 0, 1 */
	"safe_search_youtube" INTEGER NOT NULL DEFAULT 0, /* 0, 1=Moderate, 2=Strict */
	"enable_safe_search" INTEGER NOT NULL DEFAULT 0, /* 0, 1 */
	"enable_blocktime" INTEGER NOT NULL DEFAULT 0, /* 0, 1 */
	"enable_filter" INTEGER NOT NULL DEFAULT 0, /* 0, 1 */
	"enable_timequota" INTEGER NOT NULL DEFAULT 0, /* 0, 1 */
	"default_filter_config_id" INTEGER DEFAULT NULL
);

CREATE TABLE IF NOT EXISTS "schedule" (
	"id" INTEGER PRIMARY KEY AUTOINCREMENT,
	"profile_id" INTEGER NOT NULL,
	"begin_weekday" INTEGER NOT NULL, -- 0=Sunday, 6=Saturday
	"end_weekday" INTEGER NOT NULL, -- 0=Sunday, 6=Saturday
	"begin_clock" INTEGER NOT NULL, -- 0000 => 00:00
	"end_clock" INTEGER NOT NULL, -- 2359 => 23:59
	"type" INTEGER NOT NULL, -- 1=filter, 2=timequota, 3=blocktime
	FOREIGN KEY("profile_id") REFERENCES profile("id") ON DELETE CASCADE ON UPDATE CASCADE
);

CREATE TABLE IF NOT EXISTS "timequota" (
	"id" INTEGER PRIMARY KEY AUTOINCREMENT,
	"schedule_id" INTEGER NOT NULL,
	"type" INTEGER NOT NULL, -- day or week
	"quota" INTEGER DEFAULT NULL, -- minute
	FOREIGN KEY("schedule_id") REFERENCES schedule("id") ON DELETE CASCADE ON UPDATE CASCADE
);

CREATE TABLE IF NOT EXISTS "ultra_reward" (
	"id" INTEGER PRIMARY KEY AUTOINCREMENT,
	"config_group_id" INTEGER NOT NULL,
	"available" INTEGER NOT NULL, -- unix timestamp in sec
	"expired" INTEGER NOT NULL, -- unix timestamp in sec
	FOREIGN KEY("config_group_id") REFERENCES config_group("id") ON DELETE CASCADE ON UPDATE CASCADE
);

CREATE TABLE IF NOT EXISTS "config_group_hour_timespent" (
	"id" INTEGER PRIMARY KEY AUTOINCREMENT,
	"parent_id" INTEGER NOT NULL,
	"timestamp" INTEGER NOT NULL, -- unix timestamp in sec
	"last_updated" INTEGER NOT NULL DEFAULT 0, -- unix timestamp in sec
	"normal_spent" INTEGER NOT NULL DEFAULT 0, -- minute
	"reward_spent" INTEGER NOT NULL DEFAULT 0, -- minute
	UNIQUE ("parent_id", "timestamp"),
	FOREIGN KEY("parent_id") REFERENCES config_group("id") ON DELETE CASCADE ON UPDATE CASCADE
);

CREATE TABLE IF NOT EXISTS "device_hour_timespent" (
	"id" INTEGER PRIMARY KEY AUTOINCREMENT,
	"parent_id" INTEGER NOT NULL,
	"timestamp" INTEGER NOT NULL, -- unix timestamp in sec
	"last_updated" INTEGER NOT NULL DEFAULT 0, -- unix timestamp in sec
	"normal_spent" INTEGER NOT NULL DEFAULT 0, -- minute
	"reward_spent" INTEGER NOT NULL DEFAULT 0, -- minute
	UNIQUE ("parent_id", "timestamp"),
	FOREIGN KEY("parent_id") REFERENCES device("id") ON DELETE CASCADE ON UPDATE CASCADE
);

CREATE TABLE IF NOT EXISTS "config_group_minute_timespent" (
	"id" INTEGER PRIMARY KEY AUTOINCREMENT,
	"parent_id" INTEGER NOT NULL,
	"timestamp" INTEGER NOT NULL, -- unix timestamp in sec
	"last_updated" INTEGER NOT NULL DEFAULT 0, -- unix timestamp in sec
	"normal_spent" INTEGER NOT NULL DEFAULT 0, -- minute
	"reward_spent" INTEGER NOT NULL DEFAULT 0, -- minute
	UNIQUE ("parent_id", "timestamp"),
	FOREIGN KEY("parent_id") REFERENCES config_group("id") ON DELETE CASCADE ON UPDATE CASCADE
);
CREATE INDEX config_group_minute_timespent_idx ON config_group_minute_timespent ("timestamp", "parent_id", "normal_spent", "reward_spent");

CREATE TABLE IF NOT EXISTS "device_minute_timespent" (
	"id" INTEGER PRIMARY KEY AUTOINCREMENT,
	"parent_id" INTEGER NOT NULL,
	"timestamp" INTEGER NOT NULL, -- unix timestamp in sec
	"last_updated" INTEGER NOT NULL DEFAULT 0, -- unix timestamp in sec
	"normal_spent" INTEGER NOT NULL DEFAULT 0, -- minute
	"reward_spent" INTEGER NOT NULL DEFAULT 0, -- minute
	UNIQUE ("parent_id", "timestamp"),
	FOREIGN KEY("parent_id") REFERENCES device("id") ON DELETE CASCADE ON UPDATE CASCADE
);
CREATE INDEX device_minute_timespent_idx ON device_minute_timespent ("timestamp", "parent_id", "normal_spent", "reward_spent");

CREATE TABLE IF NOT EXISTS "schedule__filter_config__rel" (
	"id" INTEGER PRIMARY KEY AUTOINCREMENT,
	"schedule_id" INTEGER NOT NULL,
	"filter_config_id" INTEGER NOT NULL,
	FOREIGN KEY("schedule_id") REFERENCES schedule("id") ON DELETE CASCADE ON UPDATE CASCADE
	FOREIGN KEY("filter_config_id") REFERENCES filter_config("id") ON DELETE CASCADE ON UPDATE CASCADE
);

CREATE TRIGGER "remove_unused_schedule_filter_trigger"
	AFTER DELETE ON schedule__filter_config__rel
	FOR EACH ROW
	BEGIN
		DELETE FROM schedule WHERE id = old.schedule_id;
	END;

CREATE TABLE IF NOT EXISTS "filter_config" (
	"id" INTEGER PRIMARY KEY AUTOINCREMENT,
	"name" TEXT NOT NULL UNIQUE,
	"builtin" INTEGER NOT NULL,
	"visible" INTEGER NOT NULL DEFAULT 1, -- 1=show, 0=hide
	"color" TEXT NOT NULL,
	"type" INTEGER NOT NULL -- 1=normal, 2=whitelist, whitelist only has exception_domain
);

CREATE TRIGGER "remove_profile_default_filter_trigger"
	AFTER DELETE ON filter_config
	FOR EACH ROW
	BEGIN
		UPDATE profile SET default_filter_config_id = NULL
			WHERE default_filter_config_id = old.id;
	END;

CREATE TABLE IF NOT EXISTS "filter_config__category__rel" (
	"id" INTEGER PRIMARY KEY AUTOINCREMENT,
	"filter_config_id" INTEGER NOT NULL,
	"category_id" INTEGER NOT NULL,
	UNIQUE ("filter_config_id", "category_id")
	FOREIGN KEY("filter_config_id") REFERENCES filter_config("id") ON DELETE CASCADE ON UPDATE CASCADE
);

CREATE TABLE IF NOT EXISTS "filter_config__domain__rel" (
	"id" INTEGER PRIMARY KEY AUTOINCREMENT,
	"filter_config_id" INTEGER NOT NULL,
	"custom_domain_id" INTEGER NOT NULL,
	UNIQUE ("filter_config_id", "custom_domain_id")
	FOREIGN KEY("filter_config_id") REFERENCES filter_config("id") ON DELETE CASCADE ON UPDATE CASCADE
	FOREIGN KEY("custom_domain_id") REFERENCES custom_domain("id") ON DELETE CASCADE ON UPDATE CASCADE
);

CREATE TABLE IF NOT EXISTS "filter_config__exception_domain__rel" (
	"id" INTEGER PRIMARY KEY AUTOINCREMENT,
	"filter_config_id" INTEGER NOT NULL,
	"custom_domain_id" INTEGER NOT NULL,
	UNIQUE ("filter_config_id", "custom_domain_id")
	FOREIGN KEY("filter_config_id") REFERENCES filter_config("id") ON DELETE CASCADE ON UPDATE CASCADE
	FOREIGN KEY("custom_domain_id") REFERENCES custom_domain("id") ON DELETE CASCADE ON UPDATE CASCADE
);

CREATE TABLE IF NOT EXISTS "custom_domain" (
	"id" INTEGER PRIMARY KEY AUTOINCREMENT,
	"domain" TEXT NOT NULL,
	"category_id" INTEGER DEFAULT NULL
);
CREATE INDEX custom_domain_domain_idx ON custom_domain ('domain');
